/**
 * Created by hustcc on 18/5/10.
 * Contract: i@hust.cc
 */

const fs = require('fs');
const path = require('path');
const co = require('co');
const OSS = require('ali-oss');
const globby = require('globby');

const fileList = filePath =>
  fs.statSync(filePath).isFile() ? [ filePath ] : globby.sync([ '**/*.*' ], { cwd: filePath });

/**
 * 使用 putObject 上传
 * @param client
 * @param file
 * @param ossTarget
 */
const uploadByPutObject = async (client, file, ossTarget) => {
  const stream = fs.createReadStream(file);

  return await co(function*() {
    return Promise.resolve(
      yield client.put(ossTarget, stream),
    );
  });
};

/**
 * 使用 putStream 上传
 * @param client
 * @param file
 * @param ossTarget
 */
const uploadByPutStream = async (client, file, ossTarget) => {
  const stream = fs.createReadStream(file);

  return await co(function*() {
    return Promise.resolve(
      yield client.putStream(ossTarget, stream),
    );
  });
};

/**
 * 入口方法
 * @param filePath 上传的文件目录
 * @param config aliyun-oss 配置内容
 * @param prefix 上传到 oss 的批量文件前缀，可以用于做版本号
 * @param byStream 默认是 putObject
 */
module.exports = async (filePath, config, prefix, byStream) => {
  prefix = prefix || '';
  byStream = !!byStream;

  const client = new OSS(config);

  // 获得文件列表，并添加相对目录
  const files = fileList(filePath);

  const uploadFunc = byStream ? uploadByPutStream : uploadByPutObject;

  const r = [];

  let file;

  for (let i = 0; i < files.length; i ++) {
    file = files[i];
    // 使用 oss 上传文件 file 到 targetFile
    r.push(
      await uploadFunc(client, path.join(filePath, file), prefix + file),
    );
  }
  return r;
};
